; Example code to accompany the presentation of:
;
;             Creative Solutions for Special Data Design Problems
;
;                    Craig L. Jones, Consultant/Developer
;

; The following procedure examples are extracted from the
; FI1 Fluids Inventory system developed by Craig L. Jones.

; The code in this file is released to the public domain.

; The FI1 system itself remains copyrighted by Craig L. Jones.

;==================================================================
; Find actual count of records (w/o any empty rec)
; Return the number of records in the table not counting any empty
; "place holder" record
PROC TrueNRecords.s()
Private s
s = NImageRecords()
If s < 2 Then
  Ins
  If NImageRecords() < 2 Then s = 0 EndIf
  Del
EndIf
Return s
ENDPROC

;==================================================================
; Assign Unique Seq No. to a Detail Line
PROC DetailSeq.u()
If IsBlank([]) Then; not already assigned, so...
  v = IMAGECMAX()
  IF v = "Error" THEN
    v = 1
  ELSE
    v =INT(Max(0,v)+1)
  EndIF
  []=v
EndIf
Right
ENDPROC


;==================================================================
; Supplier Product ID Help
PROC SProdHelp.l(Opt.a)
; Note: this routine depends of the fact that the lookup table used (either
; FI1APROD, or FI1SPROD) has the fields in a certain order.  The only
; difference between APROD & SPROD is that the 2 key fields are reversed,
; and this is taken into account with indirect addressing of the
; copy-to array (i.e. by field name).

Private LineItem.s
       ,LookUpRec.r
       ,Image.s
;---------- prep ----------
Image.s = ImageNo(); Remember starting image no.
;  LineItem.s = RecNo(); remember which line we are leaving from
;  IF Opt.a = "R" AND [Order ID] <> "" THEN
;      [Order ID] = " ";  Make sure new record doesn't get closed up
;  EndIF
;  IF Opt.a = "S" AND [Product] <> "" THEN
;      [Product] = " ";  Make sure new record doesn't get closed up
;  EndIF

;---------- get to arrival ----------
  MoveTo 1
  MoveTo [Page2]
  ReSyncKey
  DownImage; go to product look-up scroll

;---------- arrive ----------
  ImageRights ReadOnly
;  Wait Table Until "F2", "Esc", "Enter"

;---------- process ----------
  RetVal = ""
  Echo Normal
  While true
    If CharWaiting() Then
      A = GetChar()
      Switch
      Case A >= 48 and A <= 122:
        Zoom Select Chr(A)+".."
      Case A = -71:  Home
      Case A = -72:  Up
      Case A = -73:  PgUp
      Case A = -79:  End
      Case A = -80:  Down
      Case A = -81:  PgDn
      Case A = 13 or A = -60:  ;Enter or Do_It!
        RetVal = "F2"
        QuitLoop
      Case A = 27:   ;Esc
        Quitloop
      EndSwitch
    EndIf
  EndWhile
  Echo Off
  ImageRights
  ; Return Retval


;---------- depart ----------
  CopyToArray LookUpRec.r
  If RetVal = "F2" Then
    Found.a = LookUpRec.r["Order ID"]; It's a good thing we can refer to
    ; this element by name, since it varies between being the 1st and 2nd
    ; element ([2] or [3])
  Else
    Found.a = ""
  EndIf

;---------- do departure --------
  UpImage; back to main table
  CtrlHome; to page 1
  If Opt.a = "L" ; Locate master record
  And Found.a > "" Then; found a valid product
    MoveTo [Color]
    Locate LookUpRec.r[4], LookUpRec.r[5], LookUpRec.r[6], LookUpRec.r[7], LookUpRec.r[8]
  EndIf
  MoveTo Image.s; to starting image

;---------- place look-up data --------
  If LookUpRec.r[3] > "" Then; found a valid product, so insert it
    Switch
    Case Opt.a = "R": ; Receivers
      [Order ID] = Found.a
      [Color]    = LookUpRec.r[4]
      [Chem]     = LookUpRec.r[5]
      [CI]       = LookUpRec.r[6]
      [Seq]      = LookUpRec.r[7]
      [Suppl]    = LookUpRec.r[8]
      [Unit]     = LookUpRec.r[9]
      [Cost/Unit]= LookUpRec.r[10]
    Case Opt.a = "S": ; Sales
      [Product]    = Found.a
      [Color]      = LookUpRec.r[4]
      [Chem]       = LookUpRec.r[5]
      [CI]         = LookUpRec.r[6]
      [Seq]        = LookUpRec.r[7]
      [Suppl]      = LookUpRec.r[8]
;      [Unit]       = LookUpRec.r[9]
      [Bulk Price] = LookUpRec.r[10]
    Case Opt.a = "M": ; Mixtures
      [I-Color]  = LookUpRec.r[4]
      [I-Chem]   = LookUpRec.r[5]
      [I-CI]     = LookUpRec.r[6]
      [I-Seq]    = LookUpRec.r[7]
      [I-Suppl]  = LookUpRec.r[8]
      [I-Name]   = Found.a
    EndSwitch
  Else
    Return False
  EndIf
  Return True
ENDPROC ; SProdHelp.l


;=========================================================================
; Find possible ways to reconcile manually entered product names sold
MESSAGE "Building Proc GenLikeSalesNames.l..."
PROC GenLikeSalesNames.l()
Private Order.s ; the sales order number
      , Line.s  ; the line item number
      , Count.s ; the number of product names that look "like" the name
                ; entered
If IsEmpty("Fi1t\\fi1_slik") Then
  Query

   Fi1t\fi1_slik | Order  | Line  |
   insert        | _order | _line |

   Fi1b\fi1shld2 | Order  | Line  |
                 | _order | _line |

  Endquery
  Do_It!
  Empty "Fi1t\\fi1_slk2"; just in case it isn't
  ClearAll
  Query; find all product names that look like the name entered

   Fi1b\fi1shld2 | Order  |  Line  | Product |
                 | Check  | Check  | _name   |

   Fi1t\fi1aprod |     Order ID     | Color  |  Chem  |   CI   |
                 | Check like _name | Check  | Check  | Check  |

   Fi1t\fi1aprod |  Seq   | Suppl  |
                 | Check  | Check  |

  Endquery
  Do_It!
  Menu {Modify} {Restructure} {Answer} ; Add column to number matches found
    Down Down Ins
    Right  "Number" Enter "S" Do_It!
  CoEditKey
  Order.s = [Order]
  Line.s = [Line]
  Count.s = 0
  Scan
    If Order.s = [Order]
    And Line.s = [Line] Then
      Count.s = Count.s + 1
      [Number] = Count.s
    Else
      Count.s = 1
      [Number] = 1
      Order.s = [Order]
      Line.s = [Line]
    EndIf
  EndScan
  Do_It!
  Menu {Tools} {More} {Add} {Answer} "Fi1t\\fi1_slk2" Enter "N"
  ; need to check for keyviol - shouldn't be any
  Return true
Else
  Return false
EndIf
ENDPROC


;==========================================================================
;
PROC MakeOrderPost.u()
; Step 1: Number the make orders the same way receivers are numbered
ClearAll
View "Sy1\\sy1ctrl"
Log.n = [Next Receiver]
CoEdit "Fi1b\\fi1make"
Scan For ([To Make] <> 0)
     And (Not IsBlank([To Make]))
     And ([Log No.] = 0)
  [Log No.] = Log.n
  Log.n = Log.n + 1
EndScan
UpImage
[Next Receiver] = Log.n
Do_It!
ClearAll
; Step 2: Append the make orders to the receivers holding main file
Query

 Fi1b\fi1make |   Log No.   |
              | _Log, Not 0 |

 Fi1b\fi1rhld | Recvr | Suppl |   Supplier Name   | Date  |
 insert       | _Log  | 200   | "(Mixed Product)" | today |

Endquery
Do_It!
ClearAll
; Step 3: Append the make orders result products to the line items holding file
Query

 Fi1b\fi1make | Color | Chem | CI  | Seq  | Suppl  | Common Name |
              | _Co   | _Ch  | _CI | _Seq | _Suppl | _Name       |

 Fi1b\fi1make | To Make  |   Log No.   |
              | _MakeAmt | _Log, Not 0 |

 Fi1b\fi1rhld2 | Recvr | Line | Order ID |   Net    | Qty |
 insert        | _Log  | 1    | _Name    | _MakeAmt | 1   |

 Fi1b\fi1rhld2 | Color | Chem | CI  | Seq  | Suppl  | Unit |
               | _Co   | _Ch  | _CI | _Seq | _Suppl | LB   |

Endquery
Do_It!
ClearAll
; Step 4: Append the make orders ingredients to the line items holding file
Query

 Fi1%mix3 |   Color   |   Chem    |    CI     |    Seq     |    Suppl     |
          | Check _Co | Check _Ch | Check _CI | Check _Seq | Check _Suppl |

 Fi1%mix3 |  Line  | Parts  | I-Color | I-Chem |  I-CI  | I-Seq  |
          | Check  | Check  | Check   | Check  | Check  | Check  |

 Fi1%mix3 | I-Suppl |  Unit  |  RO?   | Qty Avail |    Portion     |
          | Check   | Check  | Check  | Check     | Check _Portion |

 Fi1b\fi1make | Color | Chem | CI  | Seq  | Suppl  |
              | _Co   | _Ch  | _CI | _Seq | _Suppl |

 Fi1b\fi1make |                To Make                |   Log No.   |
              | _Make, calc -_Make*_Portion as "Used" | Check Not 0 |

Endquery
Do_It!
ClearAll
Query

 Answer | Line  | I-Color | I-Chem | I-CI | I-Seq |
        | _Line | _Co     | _Ch    | _CI  | _Seq  |

 Answer | I-Suppl | Log No. | Used  |
        | _Suppl  | _Log    | _Used |

 Fi1b\fi1rhld2 | Recvr |  Line   |  Net  | Qty | Color |
 insert        | _Log  | _Line+1 | _Used | 1   | _Co   |

 Fi1b\fi1rhld2 | Chem | CI  | Seq  | Suppl  | Unit |
               | _Ch  | _CI | _Seq | _Suppl | LB   |

Endquery
Do_It!
ClearAll
PostHeldReceivers.u()
ClearAll
ENDPROC ; MakeOrderPost.u
