; PopUp3 is based upon PopUp2 that is distributed with the Paradox 3.0
; Data Entry Toolkit.  PopUp3 is designed to take advantage of the Lookup-Help
; feature of Paradox in order to eliminate the need for SetPopUp.  Because the
; memory consuming arrays created by SetPopUp are not created, this will allow
; the use of a PopUp-Style Lookup Help screens in situations where memory is
; limited.  Since PopUp3 depends upon the Lookup-Help feature, any changes to
; the lookup table will be reflected dynamically, thereby eliminating the
; time it would take SetPopup to reload its arrays.  In addition, because the
; Lookup-Help feature requires that a data element must match an item in the
; lookup table, PopUp3 cannot be used in on fields that require data that
; might not exist in the lookup table.  PopUp3 allows the use of [F2] or
; [Enter] to choose an item from the list.

Createlib ("I:POPUP3")

Proc Closed Popup3(R,C,VNum,Title,Prompt1,Prompt2)

;	Private;R,			;Row position of upper-left corner of menu box
	  ;C,			;Column position of upper-left corner menu box
	  ;VNum,		;Number of items to be displayed in one menu image
	  ;Title,		;Title of popup box
	  ;Prompt1,	 ;First prompt line
	  ;Prompt2,	 ;Second prompt line
	;NItems,	  ;Number of items in menu list
	;Char,		;Keycode of last key pressed
	;MenuPos,	 ;Current (row image) position within menu
	;CIndex,	  ;Current choice index into Item
	;X,			;Counter variable
	;PrmptColr,	;Color attribute for prompt
	;BrdrColr,	;Color attribute for box border
	;ListColr,	;Color attribute for menu item list
	;SlctColr,	;Color attribute for current menu selection
	;Width,		;Width of widest item
	;OldWidth,	;Previous Width of widest item
	;Item,		;Array of items of menu
	;SearchVal	 ;Value to search list

	Echo Off					 ;Freeze workspace image

	;  Press [F1] and see if it places us into lookup-help

	Help									;press [F1]
	If helpmode()<>"LookupHelp" then		;no, it's not lookup help
		If helpmode()="Help" then Esc Endif ;it's regular help; escape from it
		Beep								;user can't press [F1] here
		Return							  ;all done with help
	Endif

	Cursor Off					;Hide blinking cursor
	Canvas Off					;Disable immediate printing to canvas

	PrmptColr = 79	  ;Top two line prompt color, yellow on red
	BrdrColr = 79	  ;Border color, yellow on red
	ListColr = 78	  ;Menu list color, white on red
	SlctColr = 30	  ;Current menu selection color, yellow on blue
	Width	= 0

	Style Attribute PrmptColr
	@ 0,0			 ;Display prompt information
	?? Spaces(80)+Prompt2+Spaces(80-Len(Prompt2))
	@ 0,0
	?? Prompt1

	Width = Max(Len(Title),Width)  ;Expand box width if title is too wide
	NItems = NRecords(table())	 ;Set number of items in lookuphelp table
	If NItems=0 then				;no records in lookup table
		Message "No Records in Lookup Table \""+Table()+"\""
		Esc
		Beep
		Return
	Endif

	If VNum > NItems Then
		VNum = NItems
	Endif

	Redraw = 0
	MenuPos = 1

	CIndex = Redraw + MenuPos			;First menu item
	LastPos = MenuPos					;Last image position is current pos.
	LastIdx = CIndex							;Last menu item index

	OldWidth = 0
	Array Item[VNum]

	Redraw = 0					;Redraw menu on first pass
	First = True
	Char = 0
	SearchVal = ""

While True

	If not(First)
	  then
		Style Attribute SlctColr
		@ MenuPos+R+2,C+2			;Highlight current selection
		?? Item[MenuPos]
		Style Attribute ListColr

	  	Char = getchar()
	Endif

	Switch
		Case Char = 0 :					;First time through, draw screen
				First = False
		Case Char > 31:								;First character search?
			MoveTo Record CIndex
			Oldrecno = [#]
			SearchVal = SearchVal + Chr(Char)
			Locate Pattern SearchVal+".."
			If Not RetVal Then					 ;Match not found
				Moveto Record Oldrecno
				Beep
			 SearchVal = ""
			Else
				If [#]<=CIndex-MenuPos+1 or [#]>=(CIndex-MenuPos+1)+VNum Then ;Is next item already visible?
					Switch				  ; No-
						Case [#] < VNum :  ;Redraw top screen
					 	Redraw = 0
					 	MenuPos = [#]
				  	Case [#] > NItems-VNum :  ;Redraw last screen
					 	Redraw = NItems-VNum
					 	MenuPos = [#] - (NItems - Vnum)
				  	Otherwise :	  ;Redraw intermediate screen
					 	Redraw = CIndex+([#]-Oldrecno)-1 ;Place item at top of menu
					 	MenuPos = 1
					Endswitch
			Else
				MenuPos = MenuPos+([#]-Oldrecno)
				Redraw = -1
			Endif
			CIndex = CIndex+([#]-Oldrecno)		  ;Update current item
		Endif
	  Case Char = -72 :					;Up
		If CIndex = 1 Then				;Already at first item?
			Beep
		Else
			If MenuPos > 1 Then			;Can move within menu image?
				MenuPos = MenuPos-1		; Yes- Move to previous item
			Else
				Redraw = CIndex-2		  ; No-  Redraw entire menu
			Endif
			CIndex = CIndex-1			 ;Update current item
		Endif
	  Case Char = -80 :					  ;Down
		If CIndex = NItems Then			 ;On last item?
			Beep
			Loop
		Else
			If MenuPos < VNum Then			;Can move within menu image?
				MenuPos = MenuPos+1			; Yes-  Move to next item
			Else
				Redraw = CIndex-VNum+1		; No- Redraw entire menu
			Endif
			CIndex=CIndex+1				  ;Update current item
		Endif
	  Case Char = -71 :						;Home
		If MenuPos <> CIndex Then			 ;Already viewing top of menu?
			Redraw = 0						 ; No-  Redraw top of menu
		Endif
		MenuPos = 1							;Position at first item
		CIndex = 1							;Select first item
		SearchVal = ""
	  Case Char = -79 :						;End
		If CIndex+VNum-MenuPos <> NItems Then ;Already viewing end of menu?
			Redraw = NItems-VNum				; No-  Redraw end of menu
		Endif
		MenuPos = VNum						;Position at bottom of menu
		CIndex = NItems						;Select last item
		SearchVal = ""
	  Case Char = -73 :						;PgUp
		If MenuPos = CIndex Then			  ;Are we within first screen?
			Beep								; Yes- Disallow PgUp
		Else
			If CIndex-MenuPos-VNum > 0 Then
				CIndex = CIndex-MenuPos-VNum+1
			Else
				CIndex = 1
			Endif
			Redraw = CIndex-1			; No-  Redraw previous page
			MenuPos = 1				 ;	  Position on that item
		Endif
		SearchVal = ""
	  Case Char = -81 :						;PgDn
		 If CIndex+VNum-MenuPos = NItems Then ;Are we within last screen?
			Beep							  ; Yes- Disallow PgDn
		 Else
			If NItems-VNum < CIndex+VNum-MenuPos Then
				CIndex = NItems-VNum+1
			Else
				CIndex = CIndex+VNum-MenuPos+1
			Endif
			Redraw = CIndex-1			; No- Redraw next page
			MenuPos = 1				 ;	 Position on that item
		 Endif
		  SearchVal = ""
	  Case Char = -60 or Char = 13 :	;[F2] Do_It! or Enter
		 Cursor Normal
		 Style
		 Esc
		 CtrlBackspace
		 Typein Item[MenuPos]				;Return selection
		 Return
	  Case Char = 27 :						;Esc
		 Cursor Normal
		 Style
		 Esc
		 Return ""							;Return null selection
	  Otherwise:							 ;Illegal key
		 Beep
	Endswitch

	If Redraw = -1 Then						;Need to redraw entire menu?
	  @ LastPos+R+2,C+2
	  ?? Item[LastPos]
	Else
	  Canvas Off						;Disable immediate canvas echo

	  Moveto Record Redraw+1

	  If Search("A",FieldType()) = 0 Then ;If field is non-alphanumeric,
		 For X From 1 To VNum			 ; convert it to a string value
			Item[X] = Strval([])		  ; before assigning it
			Width = Max(Len(Item[X]),Width)  ;Update max. width
			Skip
		 Endfor
	  Else
		 For X From 1 To VNum
			Item[X] = []
			Width = Max(Len([]),Width)
			Skip
		 Endfor
	  Endif
	  If OldWidth <> Width Then
		 OldWidth = Width
		 Style Attribute BrdrColr
		 SetMargin C
		 @ R,C						;Draw menu skeleton and initial image
		 ?? "",Fill("",Width+2),""
		  ? " ",Format("AC,W"+Strval(Width),Title)," "
		  ? "",Fill("",Width+2),""
		 For X From 1 To VNum
		  ? "",Spaces(Width+2),""
		 Endfor
		  ?  "",Fill("",Width+2),""

		 @ R+3,C+1
		 If Redraw = 0 Then				;Records above?
			?? " "
		 Else
			?? ""						 ; Yes- Show items are above
		 Endif
		 @ R+VNum+2,C+1
		 If Redraw+VNum = NItems Then	  ;Records below?
			?? " "
		 Else
			?? ""						 ; Yes- Show items are below
		 Endif
	  EndIf


	  Style Attribute ListColr
	  SetMargin C+2
	  @ R+2,C+2
	  For X from 1 to VNum			  ;Redraw entire menu box
		? Item[X]+Spaces(Width-Len(Item[X]))
	  Endfor
	  SetMargin Off

	  @ R+3,C+1
	  Style Attribute BrdrColr
	  If Redraw = 0					 ;Records above?
		Then ?? " "
		Else ?? ""					; Yes- Show items are above
	  Endif

	  @ R+VNum+2,C+1
	  If Redraw+VNum = NItems Then	  ;Records below?
		 ?? " "
	  Else
		 ?? ""						 ; Yes- Show items are below
	  Endif
	  Canvas On						 ;Enable immediate canvas echo
	  Redraw = -1
	  								;Shadow
	  Paintcanvas Attribute 8 R+VNum+4,C+1,R+VNum+4,C+Width+4
	  Paintcanvas Attribute 8 R+1,C+Width+4,R+VNum+4,C+Width+5

	Endif

	LastPos = MenuPos						 ;Update last row position
	LastIdx = CIndex						  ;Update last item index

Endwhile

Endproc
WriteLib "I:POPUP3" Popup3
Release Procs All
Release Vars All






; This version of Popup3 will concatenate the first two fields for the
; popup window, i.e., Part_No and Part_Description


Proc Closed Popup3a(R,C,VNum,Title,Prompt1,Prompt2)

;	Private;R,			;Row position of upper-left corner of menu box
	  ;C,			;Column position of upper-left corner menu box
	  ;VNum,		;Number of items to be displayed in one menu image
	  ;Title,		;Title of popup box
	  ;Prompt1,	 ;First prompt line
	  ;Prompt2,	 ;Second prompt line
	;NItems,	  ;Number of items in menu list
	;Char,		;Keycode of last key pressed
	;MenuPos,	 ;Current (row image) position within menu
	;CIndex,	  ;Current choice index into Item
	;X,			;Counter variable
	;PrmptColr,	;Color attribute for prompt
	;BrdrColr,	;Color attribute for box border
	;ListColr,	;Color attribute for menu item list
	;SlctColr,	;Color attribute for current menu selection
	;Width,		;Width of widest item
	;OldWidth,	;Previous Width of widest item
	;Item,		;Array of items of menu
	;SearchVal	 ;Value to search list

	Echo Off					 ;Freeze workspace image

	;  Press [F1] and see if it places us into lookup-help

	Help									;press [F1]
	If helpmode()<>"LookupHelp" then		;no, it's not lookup help
		If helpmode()="Help" then Esc Endif ;it's regular help; escape from it
		Beep								;user can't press [F1] here
		Return							  ;all done with help
	Endif

	Cursor Off					;Hide blinking cursor
	Canvas Off					;Disable immediate printing to canvas

	PrmptColr = 79	  ;Top two line prompt color, yellow on red
	BrdrColr = 79	  ;Border color, yellow on red
	ListColr = 78	  ;Menu list color, white on red
	SlctColr = 30	  ;Current menu selection color, yellow on blue
	FWidth	= Len(Strval([]))
	Width	= FWidth + 2
	RIGHT				;Next field over
	Width   = Width + Len(Strval([]))
	LEFT				;Back to first field

	Style Attribute PrmptColr
	@ 0,0			 ;Display prompt information
	?? Spaces(80)+Prompt2+Spaces(80-Len(Prompt2))
	@ 0,0
	?? Prompt1

	Width = Max(Len(Title),Width)  ;Expand box width if title is too wide
	NItems = NRecords(table())	 ;Set number of items in lookuphelp table
	If NItems=0 then				;no records in lookup table
		Message "No Records in Lookup Table \""+Table()+"\""
		Esc
		Beep
		Return
	Endif

	If VNum > NItems Then
		VNum = NItems
	Endif

	Redraw = 0
	MenuPos = 1

	CIndex = Redraw + MenuPos			;First menu item
	LastPos = MenuPos					;Last image position is current pos.
	LastIdx = CIndex							;Last menu item index

	OldWidth = 0
	Array Item[VNum]			;Fields to be displayed
	Array FItem[VNum]			;Field to be returned

	Redraw = 0					;Redraw menu on first pass
	First = True
	Char = 0
	SearchVal = ""

While True

	If not(First)
	  then
		Style Attribute SlctColr
		@ MenuPos+R+2,C+2			;Highlight current selection
		?? Item[MenuPos]
		Style Attribute ListColr

	  	Char = getchar()
	Endif

	Switch
		Case Char = 0 :					;First time through, draw screen
				First = False
		Case Char > 31:								;First character search?
			MoveTo Record CIndex
			Oldrecno = [#]
			SearchVal = SearchVal + Chr(Char)
			Locate Pattern SearchVal+".."
			If Not RetVal Then					 ;Match not found
				Moveto Record Oldrecno
				Beep
			 SearchVal = ""
			Else
				If [#]<=CIndex-MenuPos+1 or [#]>=(CIndex-MenuPos+1)+VNum Then ;Is next item already visible?
					Switch				  ; No-
						Case [#] < VNum :  ;Redraw top screen
					 	Redraw = 0
					 	MenuPos = [#]
				  	Case [#] > NItems-VNum :  ;Redraw last screen
					 	Redraw = NItems-VNum
					 	MenuPos = [#] - (NItems - Vnum)
				  	Otherwise :	  ;Redraw intermediate screen
					 	Redraw = CIndex+([#]-Oldrecno)-1 ;Place item at top of menu
					 	MenuPos = 1
					Endswitch
			Else
				MenuPos = MenuPos+([#]-Oldrecno)
				Redraw = -1
			Endif
			CIndex = CIndex+([#]-Oldrecno)		  ;Update current item
		Endif
	  Case Char = -72 :					;Up
		If CIndex = 1 Then				;Already at first item?
			Beep
		Else
			If MenuPos > 1 Then			;Can move within menu image?
				MenuPos = MenuPos-1		; Yes- Move to previous item
			Else
				Redraw = CIndex-2		  ; No-  Redraw entire menu
			Endif
			CIndex = CIndex-1			 ;Update current item
		Endif
	  Case Char = -80 :					  ;Down
		If CIndex = NItems Then			 ;On last item?
			Beep
			Loop
		Else
			If MenuPos < VNum Then			;Can move within menu image?
				MenuPos = MenuPos+1			; Yes-  Move to next item
			Else
				Redraw = CIndex-VNum+1		; No- Redraw entire menu
			Endif
			CIndex=CIndex+1				  ;Update current item
		Endif
	  Case Char = -71 :						;Home
		If MenuPos <> CIndex Then			 ;Already viewing top of menu?
			Redraw = 0						 ; No-  Redraw top of menu
		Endif
		MenuPos = 1							;Position at first item
		CIndex = 1							;Select first item
		SearchVal = ""
	  Case Char = -79 :						;End
		If CIndex+VNum-MenuPos <> NItems Then ;Already viewing end of menu?
			Redraw = NItems-VNum				; No-  Redraw end of menu
		Endif
		MenuPos = VNum						;Position at bottom of menu
		CIndex = NItems						;Select last item
		SearchVal = ""
	  Case Char = -73 :						;PgUp
		If MenuPos = CIndex Then			  ;Are we within first screen?
			Beep								; Yes- Disallow PgUp
		Else
			If CIndex-MenuPos-VNum > 0 Then
				CIndex = CIndex-MenuPos-VNum+1
			Else
				CIndex = 1
			Endif
			Redraw = CIndex-1			; No-  Redraw previous page
			MenuPos = 1				 ;	  Position on that item
		Endif
		SearchVal = ""
	  Case Char = -81 :						;PgDn
		 If CIndex+VNum-MenuPos = NItems Then ;Are we within last screen?
			Beep							  ; Yes- Disallow PgDn
		 Else
			If NItems-VNum < CIndex+VNum-MenuPos Then
				CIndex = NItems-VNum+1
			Else
				CIndex = CIndex+VNum-MenuPos+1
			Endif
			Redraw = CIndex-1			; No- Redraw next page
			MenuPos = 1				 ;	 Position on that item
		 Endif
		  SearchVal = ""
	  Case Char = -60 or Char = 13 :	;[F2] Do_It! or Enter
		 Cursor Normal
		 Style
		 Esc
		 CtrlBackspace
		 Typein FItem[MenuPos]				;Return selection
		 Return
	  Case Char = 27 :						;Esc
		 Cursor Normal
		 Style
		 Esc
		 Return ""							;Return null selection
	  Otherwise:							 ;Illegal key
		 Beep
	Endswitch

	If Redraw = -1 Then						;Need to redraw entire menu?
	  @ LastPos+R+2,C+2
	  ?? Item[LastPos]
	Else
	  Canvas Off						;Disable immediate canvas echo

	  Moveto Record Redraw+1

	 For X From 1 To VNum			 ; Build array of fields to display
		FItem[X] = Strval([])
		FWidth = Max(FWidth,Len(FItem[X]))
		Spacer = 2 + FWidth - Len(Strval([]))
		RIGHT		;Two blanks plus Second field
		Item[X] = FItem[X] + Fill(" ",Spacer) + Strval([])
		LEFT		;Back to first field
		Width = Max(Len(Item[X]),Width)  ;Update max. width
		Skip
	 Endfor

	 If OldWidth <> Width Then
		 OldWidth = Width
		 Style Attribute BrdrColr
		 SetMargin C
		 @ R,C						;Draw menu skeleton and initial image
		 ?? "",Fill("",Width+2),""
		  ? " ",Format("AC,W"+Strval(Width),Title)," "
		  ? "",Fill("",Width+2),""
		 For X From 1 To VNum
		  ? "",Spaces(Width+2),""
		 Endfor
		  ?  "",Fill("",Width+2),""

		 @ R+3,C+1
		 If Redraw = 0 Then				;Records above?
			?? " "
		 Else
			?? ""						 ; Yes- Show items are above
		 Endif
		 @ R+VNum+2,C+1
		 If Redraw+VNum = NItems Then	  ;Records below?
			?? " "
		 Else
			?? ""						 ; Yes- Show items are below
		 Endif
	  EndIf


	  Style Attribute ListColr
	  SetMargin C+2
	  @ R+2,C+2
	  For X from 1 to VNum			  ;Redraw entire menu box
		? Item[X]+Spaces(Width-Len(Item[X]))
	  Endfor
	  SetMargin Off

	  @ R+3,C+1
	  Style Attribute BrdrColr
	  If Redraw = 0					 ;Records above?
		Then ?? " "
		Else ?? ""					; Yes- Show items are above
	  Endif

	  @ R+VNum+2,C+1
	  If Redraw+VNum = NItems Then	  ;Records below?
		 ?? " "
	  Else
		 ?? ""						 ; Yes- Show items are below
	  Endif
	  Canvas On						 ;Enable immediate canvas echo
	  Redraw = -1
	  								;Shadow
	  Paintcanvas Attribute 8 R+VNum+4,C+1,R+VNum+4,C+Width+4
	  Paintcanvas Attribute 8 R+1,C+Width+4,R+VNum+4,C+Width+5

	Endif

	LastPos = MenuPos						 ;Update last row position
	LastIdx = CIndex						  ;Update last item index

Endwhile

Endproc
WriteLib "I:POPUP3" Popup3a
Release Procs All
Release Vars All


